local E, L, V, P, G = unpack(ElvUI);
local mPlugin = "mMediaTag"
local mMT = E:GetModule(mPlugin);
local DT = E:GetModule("DataTexts");
local addon, ns = ...

--Lua functions
local pairs = pairs
local format = format
local tonumber = tonumber
local strjoin = strjoin
local select = select
local mInsert = table.insert

--Variables
local LSM = LibStub("LibSharedMedia-3.0")
local TextureList, mIconsList = nil, nil
local mFontFlags = {
	NONE = L["NONE"],
	OUTLINE = 'Outline',
	THICKOUTLINE = 'Thick',
	MONOCHROME = '|cffaaaaaaMono|r',
	MONOCHROMEOUTLINE = '|cffaaaaaaMono|r Outline',
	MONOCHROMETHICKOUTLINE = '|cffaaaaaaMono|r Thick',
}

local function mTextureList()
	if not TextureList then
        local tmpTexture = {}
        local i = 0
        for i in pairs(mIconsList) do
            tmpTexture[i] = mIconsList[i].icon
        end
        TextureList = tmpTexture
    end
	return TextureList
end

local function mTGAtoIcon(file, i)
	return format("|T%s:16:16:0:0:128:128|t %s - %s", file, L["Icon"], i)
end

local function mGetTexturePath(file)
	return format("Interface\\AddOns\\ElvUI_mMediaTag\\media\\dock\\dockicon%s.tga", file)
end

local function SetupDockIconList()
	if not mIconsList then
        local tmpIcon = {}
        for i=1, 155, 1 do
            local path = mGetTexturePath(i)
            tmpIcon[i] = {["file"] = path, ["icon"] = mTGAtoIcon(path, i)}
        end
        mIconsList = tmpIcon
    end
end

local function OptionsDock()
	SetupDockIconList()
	
	E.Options.args.mMediaTag.args.datatext.args.docks.args = {
		dockgeneral = {
			order = 10,
			type = "group",
			name = L['General'],
			args = {
				headerdockgeneralcolor= {
					order = 0,
					type = "header",
					name = L["Color"],
				},
				docknormalstyle = {
					order = 10,
					type = 'select',
					name = L["Normal Color Style"],
					get = function(info) return E.db[mPlugin].mDock.normal.style end,
					set = function(info, value)
						E.db[mPlugin].mDock.normal.style = value
						DT:LoadDataTexts()
					end,
					values = {
						class = L["Class"],
						custom = L["Custom"],
					},
				},
				docknormalcolor = {
					type = 'color',
					order = 11,
					name = L["Custom Icon Color"],
					hasAlpha = true,
					get = function(info)
						local t = E.db[mPlugin].mDock.normal
						return t.r, t.g, t.b, t.a
					end,
					set = function(info, r, g, b, a)
						local t = E.db[mPlugin].mDock.normal
						t.r, t.g, t.b, t.a = r, g, b, a
						DT:LoadDataTexts()
					end,
				},
				dockgeneralspacer1 = {
					order = 19,
					type = "description",
					name = "\n",
				},
				dockhoverstyle = {
					order = 20,
					type = 'select',
					name = L["Hover Color Style"],
					get = function(info) return E.db[mPlugin].mDock.hover.style end,
					set = function(info, value)
						E.db[mPlugin].mDock.hover.style = value
						DT:LoadDataTexts()
					end,
					values = {
						class = L["Class"],
						custom = L["Custom"],
					},
				},
				dockhovercolor = {
					type = 'color',
					order = 21,
					name = L["Custom hover Color"],
					hasAlpha = true,
					get = function(info)
						local t = E.db[mPlugin].mDock.hover
						return t.r, t.g, t.b, t.a
					end,
					set = function(info, r, g, b, a)
						local t = E.db[mPlugin].mDock.hover
						t.r, t.g, t.b, t.a = r, g, b, a
						DT:LoadDataTexts()
					end,
				},
				dockgeneralspacer2 = {
					order = 29,
					type = "description",
					name = "\n",
				},
				dockclickstyle = {
					order = 30,
					type = 'select',
					name = L["Click Color Style"],
					get = function(info) return E.db[mPlugin].mDock.click.style end,
					set = function(info, value)
						E.db[mPlugin].mDock.click.style = value
						DT:LoadDataTexts()
					end,
					values = {
						class = L["Class"],
						custom = L["Custom"],
					},
				},
				dockclickcolor = {
					type = 'color',
					order = 31,
					name = L["Custom click Color"],
					hasAlpha = true,
					get = function(info)
						local t = E.db[mPlugin].mDock.click
						return t.r, t.g, t.b, t.a
					end,
					set = function(info, r, g, b, a)
						local t = E.db[mPlugin].mDock.click
						t.r, t.g, t.b, t.a = r, g, b, a
						DT:LoadDataTexts()
					end,
				},
				dockgeneralspacer3 = {
					order = 39,
					type = "description",
					name = "\n",
				},
				docknotificationstyle = {
					order = 40,
					type = 'select',
					name = L["Nottification Color Style"],
					get = function(info) return E.db[mPlugin].mDock.nottification.style end,
					set = function(info, value)
						E.db[mPlugin].mDock.nottification.style = value
						DT:LoadDataTexts()
					end,
					values = {
						class = L["Class"],
						custom = L["Custom"],
					},
				},
				docknotificationcolor = {
					type = 'color',
					order = 42,
					name = L["Custom nottification Color"],
					hasAlpha = true,
					get = function(info)
						local t = E.db[mPlugin].mDock.nottification
						return t.r, t.g, t.b, t.a
					end,
					set = function(info, r, g, b, a)
						local t = E.db[mPlugin].mDock.nottification
						t.r, t.g, t.b, t.a = r, g, b, a
						DT:LoadDataTexts()
					end,
				},
				headerdockgeneralsettings= {
					order = 50,
					type = "header",
					name = L["Settings"],
				},
				
				dockgeneraltip = {
					order = 56,
					type = 'toggle',
					name = L["Tooltip"],
					get = function(info) return E.db[mPlugin].mDock.tip.enable end,
					set = function(info, value)
						E.db[mPlugin].mDock.tip.enable = value
					end,
				},
				dockgeneralspacer6 = {
					order = 59,
					type = "description",
					name = "\n",
				},
				headerdockgeneralfont= {
					order = 60,
					type = "header",
					name = L["Font"],
				},
				dockgeneralfont = {
					type = 'select', dialogControl = 'LSM30_Font',
					order = 61,
					name = L["Default Font"],
					values = LSM:HashTable('font'),
					get = function(info) return E.db[mPlugin].mDock.font end,
					set = function(info, value) E.db[mPlugin].mDock.font = value; DT:LoadDataTexts(); end,
				},
				dockgeneralfontSize = {
					order = 62,
					name = L["Font Size"],
					type = 'range',
					min = 6, max = 64, step = 1,
					softMin = 8, softMax = 32,
					get = function(info) return E.db[mPlugin].mDock.fontSize end,
					set = function(info, value) E.db[mPlugin].mDock.fontSize = value; DT:LoadDataTexts(); end,
				},
				dockgeneralfontStyle = {
					type = 'select',
					order = 63,
					name = L["Font contour"],
					values = mFontFlags,
					get = function(info) return E.db[mPlugin].mDock.fontflag end,
					set = function(info, value) E.db[mPlugin].mDock.fontflag = value; DT:LoadDataTexts(); end,
				},
				
			},
		},
		dockachievment = {
			order = 20,
			type = "group",
			name = ACHIEVEMENT_BUTTON,
			args = {
				dockachievmenticon = {
					order = 1,
					type = 'select',
					name = L["Icon"],
					get = function(info) return E.db[mPlugin].mDock.textures.achievement.name end,
					set = function(info, value)
						E.db[mPlugin].mDock.textures.achievement.name = value
						E.db[mPlugin].mDock.textures.achievement.path = mIconsList[value].file
						DT:ForceUpdate_DataText("mAchievement")
					end,
					values = mTextureList(),
				},
			},
		},
		dockblizzardstore = {
			order = 30,
			type = "group",
			name = BLIZZARD_STORE,
			args = {
				dockblizzardstoreicon = {
					order = 1,
					type = 'select',
					name = L["Icon"],
					get = function(info) return E.db[mPlugin].mDock.textures.blizzardstore.name end,
					set = function(info, value)
						E.db[mPlugin].mDock.textures.blizzardstore.name = value
						E.db[mPlugin].mDock.textures.blizzardstore.path = mIconsList[value].file
						DT:ForceUpdate_DataText("mBlizzardStore")
					end,
					values = mTextureList(),
				},
			},
		},
		dockcharcter = {
			order = 40,
			type = "group",
			name = CHARACTER_BUTTON,
			args = {
				dockcharctericon = {
					order = 1,
					type = 'select',
					name = L["Icon"],
					get = function(info) return E.db[mPlugin].mDock.textures.character.name end,
					set = function(info, value)
						E.db[mPlugin].mDock.textures.character.name = value
						E.db[mPlugin].mDock.textures.character.path = mIconsList[value].file
						DT:ForceUpdate_DataText("mCharacter")
					end,
					values = mTextureList(),
				},
				dockcharcteroptioncolor = {
					order = 2,
					type = 'select',
					name = L["Text Color Styl"],
					get = function(info) return E.db[mPlugin].mDock.textures.character.color end,
					set = function(info, value)
						E.db[mPlugin].mDock.textures.character.color = value
						DT:ForceUpdate_DataText("mCharacter")
					end,
					values = {
						default = L["Default"],
						elvui = L["ElvUI"],
					},
				},
				dockcharcteroption = {
					order = 3,
					type = 'select',
					name = L["Show Text on Icon"],
					get = function(info) return E.db[mPlugin].mDock.textures.character.option end,
					set = function(info, value)
						E.db[mPlugin].mDock.textures.character.option = value
						DT:ForceUpdate_DataText("mCharacter")
					end,
					values = {
						none = L["NONE"],
						durability = L["Durability"],
						ilvl = L["Itemlevel"],
					},
				},
			},
		},
		dockcollection = {
			order = 50,
			type = "group",
			name = COLLECTIONS,
			args = {
				dockcollectionicon = {
					order = 1,
					type = 'select',
					name = L["Icon"],
					get = function(info) return E.db[mPlugin].mDock.textures.collection.name end,
					set = function(info, value)
						E.db[mPlugin].mDock.textures.collection.name = value
						E.db[mPlugin].mDock.textures.collection.path = mIconsList[value].file
						DT:ForceUpdate_DataText("mCollectionsJourna")
					end,
					values = mTextureList(),
				},
			},
		},
		dockencounter = {
			order = 60,
			type = "group",
			name = ENCOUNTER_JOURNAL,
			args = {
				dockencountericon = {
					order = 1,
					type = 'select',
					name = L["Icon"],
					get = function(info) return E.db[mPlugin].mDock.textures.encounter.name end,
					set = function(info, value)
						E.db[mPlugin].mDock.textures.encounter.name = value
						E.db[mPlugin].mDock.textures.encounter.path = mIconsList[value].file
						DT:ForceUpdate_DataText("mEncounterJournal")
					end,
					values = mTextureList(),
				},
			},
		},
		dockguild = {
			order = 70,
			type = "group",
			name = GUILD_AND_COMMUNITIES,
			args = {
				dockguildicon = {
					order = 1,
					type = 'select',
					name = L["Icon"],
					get = function(info) return E.db[mPlugin].mDock.textures.guild.name end,
					set = function(info, value)
						E.db[mPlugin].mDock.textures.guild.name = value
						E.db[mPlugin].mDock.textures.guild.path = mIconsList[value].file
						DT:ForceUpdate_DataText("mGuild")
					end,
					values = mTextureList(),
				},
			},
		},
		docklfd = {
			order = 80,
			type = "group",
			name = DUNGEONS_BUTTON,
			args = {
				docklfdicon = {
					order = 10,
					type = 'select',
					name = L["Icon"],
					get = function(info) return E.db[mPlugin].mDock.textures.lfd.name end,
					set = function(info, value)
						E.db[mPlugin].mDock.textures.lfd.name = value
						E.db[mPlugin].mDock.textures.lfd.path = mIconsList[value].file
						DT:ForceUpdate_DataText("mLFDTool")
					end,
					values = mTextureList(),
				},
				docklfdgreatvault = {
					order = 20,
					type = 'toggle',
					name = L["Great Vault"],
					desc = L["Show Greaut Vault infos in the Tooltip and opens the Great Vault"],
					get = function(info)
						return E.db[mPlugin].mDock.settings.lfd.greatvault
					end,
					set = function(info, value)
						E.db[mPlugin].mDock.settings.lfd.greatvault = value
					end,
				},
				docklfdaffixes = {
					order = 40,
					type = 'toggle',
					name = L["Weekly Affixes"],
					desc = L["Shows the Weekly Affixes."],
					get = function(info)
						return E.db[mPlugin].mDock.settings.lfd.affix
					end,
					set = function(info, value)
						E.db[mPlugin].mDock.settings.lfd.affix = value
					end,
				},
				docklfdkeystone = {
					order = 50,
					type = 'toggle',
					name = L["Tooltip Keystone"],
					desc = L["Shows your Keystone in the tooltip."],
					get = function(info)
						return E.db[mPlugin].mDock.settings.lfd.keystone
					end,
					set = function(info, value)
						E.db[mPlugin].mDock.settings.lfd.keystone = value
					end,
				},
				docklfddifficulty = {
					order = 60,
					type = 'toggle',
					name = L["Difficulty Text"],
					get = function(info)
						return E.db[mPlugin].mDock.settings.lfd.difficulty
					end,
					set = function(info, value)
						E.db[mPlugin].mDock.settings.lfd.difficulty = value
						DT:ForceUpdate_DataText("mLFDTool")
					end,
				},
			},
		},
		dockmainmenu = {
			order = 90,
			type = "group",
			name = MAINMENU_BUTTON,
			args = {
				dockmainmenuicon = {
					order = 1,
					type = 'select',
					name = L["Icon"],
					get = function(info) return E.db[mPlugin].mDock.textures.mainmenu.name end,
					set = function(info, value)
						E.db[mPlugin].mDock.textures.mainmenu.name = value
						E.db[mPlugin].mDock.textures.mainmenu.path = mIconsList[value].file
						DT:ForceUpdate_DataText("mMainMenu")
					end,
					values = mTextureList(),
				},
				dockmainmenucolor = {
					order = 2,
					type = 'select',
					name = L["Text Color Styl"],
					get = function(info) return E.db[mPlugin].mDock.textures.mainmenu.color end,
					set = function(info, value)
						E.db[mPlugin].mDock.textures.mainmenu.color = value
						DT:ForceUpdate_DataText("mMainMenu")
					end,
					values = {
						default = L["Default"],
						elvui = L["ElvUI"],
					},
				},
				dockmainmenuoption = {
					order = 3,
					type = 'select',
					name = L["Show Text on Icon"],
					get = function(info) return E.db[mPlugin].mDock.textures.mainmenu.option end,
					set = function(info, value)
						E.db[mPlugin].mDock.textures.mainmenu.option = value
						if value == "fps" then
							E.db[mPlugin].mDock.textures.mainmenu.text = "FPS"
						else
							E.db[mPlugin].mDock.textures.mainmenu.text = "MS"
						end
						DT:ForceUpdate_DataText("mMainMenu")
					end,
					values = {
						none = L["NONE"],
						fps = L["FPS"],
						ms = L["MS"],
					},
				},
				dockmainmenutext = {
					order = 4,
					name = L["Text"],
					type = 'input',
					width = 'smal',
					get = function() return E.db[mPlugin].mDock.textures.mainmenu.text end,
					set = function(info, value)
						E.db[mPlugin].mDock.textures.mainmenu.text = value
						DT:ForceUpdate_DataText("mMainMenu")
					end,
				},
			},
		},
		dockquest = {
			order = 100,
			type = "group",
			name = QUESTLOG_BUTTON,
			args = {
				dockquesticon = {
					order = 1,
					type = 'select',
					name = L["Icon"],
					get = function(info) return E.db[mPlugin].mDock.textures.quest.name end,
					set = function(info, value)
						E.db[mPlugin].mDock.textures.quest.name = value
						E.db[mPlugin].mDock.textures.quest.path = mIconsList[value].file
						DT:ForceUpdate_DataText("mQuest")
					end,
					values = mTextureList(),
				},
			},
		},
		dockspellbook = {
			order = 110,
			type = "group",
			name = SPELLBOOK_ABILITIES_BUTTON,
			args = {
				dockspellbookicon = {
					order = 1,
					type = 'select',
					name = L["Icon"],
					get = function(info) return E.db[mPlugin].mDock.textures.spellbook.name end,
					set = function(info, value)
						E.db[mPlugin].mDock.textures.spellbook.name = value
						E.db[mPlugin].mDock.textures.spellbook.path = mIconsList[value].file
						DT:ForceUpdate_DataText("mSpellBook")
					end,
					values = mTextureList(),
				},
			},
		},
		docktalent = {
			order = 120,
			type = "group",
			name = TALENTS_BUTTON,
			args = {
				docktalenticon = {
					order = 1,
					type = 'select',
					name = L["Icon"],
					get = function(info) return E.db[mPlugin].mDock.textures.talent.name end,
					set = function(info, value)
						E.db[mPlugin].mDock.textures.talent.name = value
						E.db[mPlugin].mDock.textures.talent.path = mIconsList[value].file
						DT:ForceUpdate_DataText("mTalent")
					end,
					values = mTextureList(),
				},
			},
		},
		dockitemlevel = {
			order = 130,
			type = "group",
			name = L["Itemlevel"],
			args = {
				dockitemlevelicon = {
					order = 1,
					type = 'select',
					name = L["Icon"],
					get = function(info) return E.db[mPlugin].mDock.textures.itemlevel.name end,
					set = function(info, value)
						E.db[mPlugin].mDock.textures.itemlevel.name = value
						E.db[mPlugin].mDock.textures.itemlevel.path = mIconsList[value].file
						DT:ForceUpdate_DataText("mItemLevel")
					end,
					values = mTextureList(),
				},
				dockitemleveltext = {
					order = 2,
					name = L["Text"],
					type = 'input',
					width = 'smal',
					get = function() return E.db[mPlugin].mDock.textures.itemlevel.text end,
					set = function(info, value)
						E.db[mPlugin].mDock.textures.itemlevel.text = value
						DT:ForceUpdate_DataText("mItemLevel")
					end,
				},
			},
		},
		dockfriends = {
			order = 140,
			type = "group",
			name = L["Friends"],
			args = {
				dockfriendsicon = {
					order = 1,
					type = 'select',
					name = L["Icon"],
					get = function(info) return E.db[mPlugin].mDock.textures.friends.name end,
					set = function(info, value)
						E.db[mPlugin].mDock.textures.friends.name = value
						E.db[mPlugin].mDock.textures.friends.path = mIconsList[value].file
						DT:ForceUpdate_DataText("mFriends")
					end,
					values = mTextureList(),
				},
			},
		},
		dockdurability = {
			order = 150,
			type = "group",
			name = L["Durability"],
			args = {
				dockdurabilityicon = {
					order = 1,
					type = 'select',
					name = L["Icon"],
					get = function(info) return E.db[mPlugin].mDock.textures.durability.name end,
					set = function(info, value)
						E.db[mPlugin].mDock.textures.durability.name = value
						E.db[mPlugin].mDock.textures.durability.path = mIconsList[value].file
						DT:ForceUpdate_DataText("mDurability")
					end,
					values = mTextureList(),
				},
				dockdurabilitycolor = {
					order = 2,
					type = 'select',
					name = L["Text Color Styl"],
					get = function(info) return E.db[mPlugin].mDock.textures.durability.color end,
					set = function(info, value)
						E.db[mPlugin].mDock.textures.durability.color = value
						DT:ForceUpdate_DataText("mDurability")
					end,
					values = {
						default = L["Default"],
						elvui = L["ElvUI"],
					},
				},
			},
		},
		dockfpsms = {
			order = 160,
			type = "group",
			name = L["FPS / MS"],
			args = {
				dockfpsmsicon = {
					order = 1,
					type = 'select',
					name = L["Icon"],
					get = function(info) return E.db[mPlugin].mDock.textures.fpsms.name end,
					set = function(info, value)
						E.db[mPlugin].mDock.textures.fpsms.name = value
						E.db[mPlugin].mDock.textures.fpsms.path = mIconsList[value].file
						DT:ForceUpdate_DataText("mFPSMS")
					end,
					values = mTextureList(),
				},
				dockfpsmscolor = {
					order = 2,
					type = 'select',
					name = L["Text Color Styl"],
					get = function(info) return E.db[mPlugin].mDock.textures.fpsms.color end,
					set = function(info, value)
						E.db[mPlugin].mDock.textures.fpsms.color = value
						DT:ForceUpdate_DataText("mFPSMS")
					end,
					values = {
						default = L["Default"],
						elvui = L["ElvUI"],
					},
				},
				dockfpsmsoption = {
					order = 3,
					type = 'select',
					name = L["Show Text on Icon"],
					get = function(info) return E.db[mPlugin].mDock.textures.fpsms.option end,
					set = function(info, value)
						E.db[mPlugin].mDock.textures.fpsms.option = value
						if value == "fps" then
							E.db[mPlugin].mDock.textures.fpsms.text = "FPS"
						else
							E.db[mPlugin].mDock.textures.fpsms.text = "MS"
						end
						DT:ForceUpdate_DataText("mFPSMS")
					end,
					values = {
						fps = L["FPS"],
						ms = L["MS"],
					},
				},
				dockfpsmstext = {
					order = 4,
					name = L["Text"],
					type = 'input',
					width = 'smal',
					get = function() return E.db[mPlugin].mDock.textures.fpsms.text end,
					set = function(info, value)
						E.db[mPlugin].mDock.textures.fpsms.text = value
						DT:ForceUpdate_DataText("mFPSMS")
					end,
				},
			},
		},
		docknottification = {
			order = 200,
			type = "group",
			name = L["Nottification"],
			args = {
				docknottificationicon = {
					order = 1,
					type = 'select',
					name = L["Icon"],
					get = function(info) return E.db[mPlugin].mDock.textures.nottification.name end,
					set = function(info, value)
						E.db[mPlugin].mDock.textures.nottification.name = value
						E.db[mPlugin].mDock.textures.nottification.path = mIconsList[value].file
						DT:LoadDataTexts()
					end,
					values = mTextureList(),
				},
				docknottificationiconsize = {
					order = 2,
					name = L["Icon size"],
					type = "range",
					min = 16, max = 128, step = 2,
					get = function(info)
						return E.db[mPlugin].mDock.nottification.size
					end,
					set = function(info, value)
						E.db[mPlugin].mDock.nottification.size = value
						DT:LoadDataTexts()
					end,
				},
				docknottificationspacer1 = {
					order = 3,
					type = "description",
					name = "\n",
				},
				docknottificationiconflash = {
					order = 4,
					type = 'toggle',
					name = L["Icon flash"],
					get = function(info)
						return E.db[mPlugin].mDock.nottification.flash
					end,
					set = function(info, value)
						E.db[mPlugin].mDock.nottification.flash = value
					end,
				},
			},
		},
	}
end

mInsert(ns.Config, OptionsDock)